/* SPDX-License-Identifier: (BSD-3-Clause OR GPL-2.0-only)*/
/* Copyright(c) 2019 - 2022 Intel Corporation */
#ifndef ADF_VDCM_MIGRATION_H
#define ADF_VDCM_MIGRATION_H

#define ADF_VQAT_MSTATE_MAGIC	0xADF5CAEA
#define ADF_VQAT_MSTATE_VERSION	0x1

#define ADF_VQAT_MSTATE_ETRB_IDS	"ETRBAR"
#define ADF_VQAT_MSTATE_MISCB_IDS	"MISCBAR"
#define ADF_VQAT_MSTATE_EXTB_IDS	"EXTBAR"
#define ADF_VQAT_MSTATE_SECTION_NUM	3

struct adf_vqat_mstate_preh {
	u32 magic;
	u32 version;
	u16 preh_len;
	u16 n_sects;
	u32 size;
};

struct adf_vqat_mstate_sect_info {
	void *sect;
	u32 size;
};

struct adf_vqat_mstate_vreginfo {
	void *addr;
	u32 size;
};

static inline
void adf_vqat_mstate_preamble_init(struct adf_vqat_mstate_preh *preamble)
{
	preamble->magic = ADF_VQAT_MSTATE_MAGIC;
	preamble->version = ADF_VQAT_MSTATE_VERSION;
	preamble->preh_len = sizeof(*preamble);
	preamble->size = 0;
	preamble->n_sects = 0;
}

/* default preambles checker */
static inline
int adf_vqat_mstate_preamble_checker(struct adf_vqat_mstate_preh *preamble,
				     void *opaque)
{
	if (preamble->magic != ADF_VQAT_MSTATE_MAGIC ||
	    preamble->version > ADF_VQAT_MSTATE_VERSION) {
		pr_err("%s: unrecognized vqat state, magic=0x%x,version=0x%x, hlen=%u\n",
		       __func__, preamble->magic,
		       preamble->version, preamble->preh_len);
		return -EINVAL;
	}

	return 0;
}

void *adf_vqat_mstate_mgr_new(u8 *buf, u32 size, void *parent_sec);
void adf_vqat_mstate_mgr_reinit(void *mgr, u8 *buf, u32 size,
				void *parent_sec);
void adf_vqat_mstate_mgr_destroy(void *mgr);
struct adf_vqat_mstate_preh *
adf_vqat_mstate_preamble_add(void *mgr);
int adf_vqat_mstate_preamble_update(void *mgr,
				    struct adf_vqat_mstate_preh *preamble);
struct adf_vqat_mstate_preh *
adf_vqat_mstate_preamble_get(void *mgr);
void *adf_vqat_mstate_sect_add(void *mgr,
			       const char *id,
			       int (*populate)(void *sub_sects_mgr, u8 *buf,
					       u32 size, void *opaque),
			       void *opaque);
int
adf_vqat_mstate_mgr_scan(void *mgr,
			 int (*pre_checker)(struct adf_vqat_mstate_preh *,
					    void *),
			 void *opaque);
void *adf_vqat_mstate_sect_lookup(void *mgr,
				  const char *id,
				  int (*action)(void *sub_sects_mgr, u8 *buf,
						u32 size, void *opaque),
				  void *opaque);
int adf_vqat_mstate_state_size(void *mgr);
int adf_vqat_mstate_sect_info_get(void *sect,
				  struct adf_vqat_mstate_sect_info *sinfo);
#endif
